C
C     -----------------------------------
      SUBROUTINE Gravity_Tide (X,Y,ZPR)
ccc      SUBROUTINE Gravity_Tide (X,Y,H,ZPR)
C Potential * on earth elliptical surface, Mathews et al., 1995
C     -----------------------------------
C
C We follow Wahr (1981) for the treatment of Delta-factors and
C equatorial reference. To obtain the Delta-factors in the Dehant-system
C we must divide through the rigid-earth gravity tide on the spheroid.
C See tap/t/p/urtapu5.f lines 100++
C
C This version does not regard locations away from the reference ellipsoid.
C The ellipsoid is intrinsic in Wahr's parameters, except for the
C direct effect. RRN(i) is prepared, mark ccc
C HGS 2014-07-20
C
C This version neglects inertial effects. To obtain the inertial effects,
C compute vertical and east displacements ( V resp. E) 
C take d²V/dt² and omega*cos(lat)*dE/dt 
C and don't forget that our frequency arrays are omega's in rad/h
C
C Pendant to Displacement_Tide. 
C
C ZPR(1200) - complex*16
C             gravity spectrum, still in uGal
C X,Y       - real*8
C             Site longitude, colatitude (radians)
C
C
C Entry SP_Gravity_Tide (X,Y,ZPRSP,ZPHSP)
C 
C ZPRSP     - complex*16 
C             gravity value, no phase
C ZPHSP     - complex*16
C             the phase: ZPR=ZPRSP*ZPHSP
C 
      IMPLICIT        REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2       IAA, IATR(0:6), IAA_P(0:9)
      INTEGER         ICO(2:4,0:4)
      REAL*8          P(2:4,0:4), RRP(2:6)
     &,               CORDH(2:4,0:4), CORDL(2:4,0:4), CORDK(2:4,0:4)
      REAL*8          CORDG(0:2),WG0(0:2)
      REAL*8          WLP0(0:2),WWP0(0:2),WZ0(0:2),WKP0(0:2),WSLP(0:2)
      REAL*8          prat(2:4,0:4)
ccc      REAL*8          rrn(2:6)
      COMPLEX*16      ZLO(0:4), ZPR(1200), ZPRSP, DELTAF, deltad
      LOGICAL         QMSG,QQ,QSP,QUICK,QNDSPLCM,QNM(0:4,0:4)

      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
      COMMON /MPAR/   PI,RAD
C
C Earth parameters:   Mean radius, equat.radius, equat.gravity, rot.velocity,
C                     Basic Love numbers:
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE,BG,RHOW
C
C Near diurnal resonance (NDR) parameters:
      COMMON /COREP/  ONDR,QNDR,FREFC
     &,               SEHNDR,SELNDR,SEKNDR,ZOHNDR,ZOLNDR,ZOKNDR
C
C Love number ratios for spherical harmonic degrees and orders w r t  n=2, m=1
      COMMON /SHDOP/  CORDH, CORDL, CORDK
C
C Spherical harmonic coupling parameters; Wahr-model
      COMMON /WAHRP/  WWP0,WLP0,WZ0,WY,WWP1,WLP1,WZ1,WS3,WKP0,WSLP
     &,               WT(0:5)
C
C Parameters for the Mathews Buffett Shapiro GRAVity model
      COMMON /CMBSG/  DELTA_O1,DELTA_PRIME,CORDG,SEGNDR,WG0,WG1

      SAVE
      DATA  ZI/(0.0D0,1.0D0)/, QMSG/.TRUE./, QNDSPLCM/.FALSE./
     &,     QNM/24*.true./, hsite/0.d0/

      QSP=.FALSE.
      GOTO 1

      ENTRY SP_Gravity_Tide (X,Y,IAA_P,ZPRSP,ZPHSP)
      QSP=.TRUE.

    1 QUICK=.FALSE.
      IF (Y/RAD.LT.1.D-3.OR.Y/RAD.GT.179.99D0) THEN
         PRINT '(A,F10.4)','0<GrTide>W> Invalid colatitude :'
     &,   Y/RAD,'. Adjusted.'
         Y=DMIN1(179.99*RAD,DMAX1(1.D-3*RAD,Y))
      ENDIF
      FRENDR=(1.D0+1.D0/ONDR)*EROT
      ZFRNDR=CMPLX(FRENDR,FRENDR/QNDR/2.D0)
      TMIN=1.D10

c      req=get_epar('REQUATOR')
      req=re
      call geog_to_geoc (hsite,x,pi/2.d0-y,res,xlo,ycl)
      rratio=res/req
      colatg=y
      colatc=pi/2.d0-ycl

      IF (QMSG) THEN
         PRINT '(A,2F12.6)'
     &,        '0<GrTide>>> Site longitude / colatitude:',X/RAD,Y/RAD
         PRINT '(A)',' <GrTide>>> (P(N,M), M=0,N):'
      ENDIF
      
      DO 20 N=2,4
         DO 21 M=0,N
            ICO(N,M)=0
            P(N,M)=PNM(N,M,Y)
 21      CONTINUE
         IF (QMSG) PRINT '(I3,5F11.4)'
     &,                   N,(P(N,M),M=0,N)
 20      CONTINUE

      IF (QMSG) then
         print '(a,1p,2e15.6)'
     &,        '0<GrTide>>> Radii equator, geograph ',req,res
         print '(a,3f12.6)'
     &,        ' <GrTide>>> Site colat geocentr. geograph. ratio S/Eq ='
     &,        colatc/rad,colatg/rad,rratio
         print '(a)',' <GrTide>>> (PRAT(N,M), M=0,N):'
      endif
      do i=2,4
         do j=0,i
            prat(i,j)=pnm(i,j,colatg)/pnm(i,j,colatc)
            IF (QMSG) 
     &         print '(a,2i3,f12.6)',' <GrTide>>> ',i,j,prat(i,j)
         enddo
      enddo

      DO 22 M=0,4
         DM=DFLOAT(M)
 22      ZLO(M)=EXP(ZI*DM*X)
C                          (N.B. use CDEXP instead of EXP if available)
C
      XX=X
      YY=Y
ccc     rs=re+h

      GN=1.d-9 ! converts m/s^2 into nm/s^2

c      print '(a,1p,4d12.4)',' <GrTide>>> rr,gn,y:',rr,gn,y/rad,p(2,1)
c      print *,'se_, zo_kndr:',sekndr,zokndr

      DO I=2,6
         RRP(I)=-DFLOAT(I)/RE  ! the minus is because  Acc = - grad Pot
ccc         rrn(i)=(rs/re)**i     ! Newtonian, for rs=re+alt
ccc         rrk(i)=(re/rs)**(i+1) ! for the k-part
      ENDDO
      CALL BLMULT('NO')
C
C ------------------------- Step through the astron.spectrum -->>>
      IV=0
c23456789012345678901234567890123456789012345678901234567890123456789012
      print '(a)'
     &,   ' <GrTide>>>  N M   Real__Delta__Imag     Real__Dehant__Imag'
c                      iiii12345678901234567890xx12345678901234567890
c           <GrTide>>>  2 0   1.15623   0.00000     1.14488   0.00000

ccc      open (unit=91,file='gelaelle.dmp',form='formatted')

 100  continue

      NV_O1=NR_OF_TIDESYMBOL ('O1',.FALSE.)
      IF (QSP) THEN
         IV=NR_OF_TIDE(IAA_P,9)
      ELSE
         IV=IV+1
         IF (IV.GT.NV) GOTO 101
      ENDIF
      N=IAA(0,IV)
      ZPHA=EXP(ZI*PHA(IV))
C
C Discard sph.harm.degree > 4
      IF (N.GT.4.OR.N.LT.2) THEN
         IF (QSP) THEN
            ZPRSP=(0.0D0,0.0D0)
            RETURN
         ELSE
            ZPR(IV)=(0.0D0,0.0D0)
         ENDIF
         GOTO 100
      ENDIF
      M=IAA(1,IV)
      ICO(N,M)=ICO(N,M)+1
      urat = rratio**(iaa(0,iv)-1)/prat(n,m)
c      urat = rratio**iaa(0,iv)/prat(n,m)
      IF (N.EQ.2) THEN
         IF (M.EQ.1) THEN
            FDP=0.D0
            ZCLQCK=1.D0 + (SEGNDR+WG1*P(2,M))*(FRE(IV)-FREFC)
     &                                       /(FRE(IV)-ZFRNDR)
            if (iv.eq.nv_o1.and.qnm(n,m)) then
               deltaf=DELTA_O1*ZCLQCK*(CORDG(M)
     &                +WG0(M)*P(2,0))+FDP
               deltad=deltaf*urat
               print '(a,2i2,2(2f10.5,2x))'
     &,              ' <GrTide>D> ',n,m,deltaf,deltad
               qnm(n,m)=.false.
            endif
         ELSEIF (M.EQ.0) THEN
            FDP=DELTA_PRIME
            ZCLQCK=(1.0D0,0.0D0)
         ELSE
            FDP=0.D0
            ZCLQCK=(1.0D0,0.0D0)
         ENDIF
         ZPV=RRP(2)*(P(2,M)*DELTA_O1*ZCLQCK*(CORDG(M)
     &                     +WG0(M)*P(2,0))+FDP)
         if (qnm(n,m).and.m.ne.1) then
            deltaf=DELTA_O1*ZCLQCK*(CORDG(M)
     &            +WG0(M)*P(2,0))+FDP
            deltad=deltaf*urat
         endif

cc
cc             +P(2,M)*(rrn(2)-1.d0)
c
c         ZPV=RRP(2)*(P(2,M)*DELTA_O1*ZCLQCK*(CORDG(M)
c     &                     +WG0(M)*P(2,0))+FDP)
c
      ELSE
         ZPV=P(N,M)*RRP(N)*(
     &          DFLOAT(N)  
     &                   +2*BL(1)*CORDH(N,M)
     &         -DFLOAT(N+1)*BL(3)*CORDK(N,M))  /DFLOAT(N)
         if (qnm(n,m)) then 
            deltaf=(
     &          DFLOAT(N)  
     &                   +2*BL(1)*CORDH(N,M)
     &         -DFLOAT(N+1)*BL(3)*CORDK(N,M))  /DFLOAT(N)
            deltad=deltaf*urat
         endif
c2345c789012345678901234567890123456789012345678901234567890123456789012.......|
      ENDIF
ccc      xamp=amp(iv)/9.798529D+00
ccc      write (91,'(3h % ,8i3,1p,e14.6,0p,4f12.6)')
ccc     & (iaa(ii,iv),ii=0,7),fre(iv)*12/pi,amp(iv)
ccc     &,xamp*1d2,p(n,m),xamp*p(n,m)*1.d2

      if (qnm(n,m).and.(m.ne.1.or.n.ne.2)) then
         print '(a,2i2,2(2f10.5,2x))'
     &,        ' <GrTide>D> ',n,m,deltaf,deltad
         qnm(n,m)=.false.
      endif
      QUICK=.TRUE.
      IF (QSP) THEN
         ZPRSP=AMP(IV)*ZPV*ZLO(M)/GN
         ZPHSP=ZPHA
         GOTO 101
      ELSE
         ZPV=AMP(IV)*ZPV*ZLO(M)*ZPHA/GN
         ZPR(IV)=ZPV
         TMIN=DMIN1(TMIN,ABS(ZPV))
      ENDIF
      GOTO 100

 101  CONTINUE

cc      close (91)

      IF (QMSG) THEN
         PRINT '(A,1P,D10.2,A)'
     &,' <GrTide>>> Min. tide amplitudes (gravity) :'
     &, TMIN,' nm/s^2'
         PRINT '(A)',' <GrTide>>>'
     &,' <GrTide>>> Number of tides in the different bands:'
     &,' <GrTide>>>  N / M=0    1    2    3    4'
         DO 111 N=2,4
 111     PRINT '(12H <GrTide>>> ,I2,1X,5I5)', N,(ICO(N,M),M=0,N)
         PRINT '(11H <GrTide>>>)'
         IF (QSP) QMSG=.FALSE.
      ENDIF
      RETURN

      ENTRY Gravity_Tide_MSG (QQ)
      QMSG=QQ
      RETURN

      END

      BLOCK DATA BMBSG
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 CORDG(0:2),WG0(0:2)
C
C Parameters for the Mathews Buffett Shapiro GRAVity model
      COMMON /CMBSG/  DELTA_O1,DELTA_PRIME,CORDG,SEGNDR,WG0,WG1
      
      DATA DELTA_O1/1.1549D0/, DELTA_PRIME/-0.0001D0/
      DATA CORDG/1.001385D0,1.0D0,1.000793D0/
      DATA WG0/3*-0.00026D0/, WG1/2.3D-6/
      DATA SEGNDR/0.00055D0/
      END


      
