      SUBROUTINE Print_Major_Tides (many,ZDR,columns,sort_column)
c BTIDE * print a 3-component spectrum
c
c MANY         - integer    - the number of lines to print
c ZDR(1200,*)  - complex*16 - the effects' components
c COLUMNS      - integer    - the number of components, 1 or 3
c                             E.g. TILT will use columns 2 and 3
c                             irregardless whether column 1 has been
c                             filled with GRAVity or not.
c Sort_Column  - integer    - What column to sort on 
c                             (CDABS(ZDR(.,Sort_Column)) in falling order). 
c
c N.B. Options that can be set through entry Print_Tides_Opt
c                              
c
      real*8          as(1200), amp,pha,fre, pi,rad, dlon,slon
     &,               xx,phaeff, phapot, xs, a
      integer         ip(1200), iq(1200), columns, sort_column
      integer*2       IFR(0:9,1200)
      complex*16      ZDR(1200,*), Z, ZP, ZV(3), DPOLAR, ZPOT
      character       SYMB*4, opt*(*), line*142
      logical         QP, QO, QPP, QP1, qor_index, q_index, qphint
      
      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IFR
      COMMON /MPAR/   PI,RAD
      
      DATA        QP/.FALSE./, QO/.FALSE./, QPP/.false./, xs/0.0d0/
     &,           QPHINT/.false./, QP1/.false./, slon/0.0d0/

      K      =    sort_column
      DO 211      I   =  1,NV
211   AS(I)  =    ABS    (ZDR(I,K))
      CALL        DSORT  (AS,NV,IP)
      GOTO        1

      entry Print_Sorted_Tides (many,ZDR,columns,iq)
c BTIDE * print a 3-component sorted spectrum
c
      do 101      i=1,many
101   ip(i)  =    iq(i)

1     print '(a,l2/4a)','0<PrMTid>>> Sorted tides spectrum, qp=:',qp
     &,'   ________Astr.arg.numbers____ SYMB     Freq.      '
     &,('Ampl.      Phase    ',i=1,columns)

      ZP     =    (1.0d0, 0.0d0)
      DO 103      JP  =  1,many
      I      =    IP(JP)
      CALL        Symbol_Of_Tide   (IFR(0,I),SYMB)
      IF (QP)     ZP=CDEXP((0.0d0,-1.0d0)*PHA(I))
      DO 104      J=1,columns
      ZV(j)  =    zdr(i,j)*zp
      IF (QO)     ZV(J)=DCONJG(ZV(J))
104   CONTINUE
 
c The following (in lower-case) was added 2018-01-26
c in order to evaluate the phase relations of tilt w.r.t. potential
c Option parsing rewritten, new options +P +Q -L
c Options P and O are compatible with +P and +O, respectively.
c
      dlon=0.d0
      if (qpp) then
         itc=columns
         if (qp1) then
            if (columns.eq.1) then
               itc=2
               zv(2)=zv(1)
            endif
            zv(1)=amp(i)*cdexp((0.0d0,1.0d0)*pha(i))
         endif

         write (line, 1002) (IFR(J,I),J=0,9), SYMB
     &,               fre(i)/rad/15.d0
     &,              (dpolar(zv(J),0.0d0),J=1,itc)
         
         if (qphint) dlon=ifr(1,i)*slon/rad
         
         zpot=amp(i)*cdexp((0.0d0,1.0d0)*PHA(I))
         phapot=dimag(dpolar(zpot,180.d0))

         phaeff=dimag(dpolar(zv(1),0.0d0))
         phaeff=dmod(phaeff-phapot-dlon+540.d0,360.d0)-180.d0

         if (qphint) then
            a=cdabs(zv(1))
            if (a.lt.1d-20) phaeff=0.d0
            write (line(111:), '(i4)') idnint(phaeff/90.)
         else
            write (line(111:), '(f10.3)') phaeff
         endif
         
         if (itc.gt.1) then
         phaeff=dimag(dpolar(zv(2),0.0d0))
         phaeff=dmod(phaeff-phapot-dlon+540.d0,360.d0)-180.d0
         if (qphint) then
            a=cdabs(zv(2))
            if (a.lt.1d-20) phaeff=0.d0
            write (line(116:), '(i4)') idnint(phaeff/90.)
         else
            write (line(121:), '(f10.3)') phaeff
         endif
         endif

         if (columns.gt.2) then
         phaeff=dimag(dpolar(zv(3),0.0d0))
         phaeff=dmod(phaeff-phapot-dlon+540.d0,360.d0)-180.d0
         if (qphint) then
            write (line(121:), '(i4)') idnint(phaeff/90.)
         else
            write (line(131:), '(f10.3)') phaeff
         endif
         endif

         print '(a)',trim(line)
      else
         PRINT 1002, (IFR(J,I),J=0,9), SYMB
     &,               fre(i)/rad/15.d0
     &,              (dpolar(zv(J),0.0d0),J=1,columns)
      endif
 103  CONTINUE

1002  FORMAT      (1H ,10I3,1x,a4,1x,f11.7,4(1P,D12.4,0P,F8.2))
      print '(a,l2)',' <PrMtid>>> Oceanographers phase:',qo
      print '(a)'
     &,' _____________________________________________________________'
     &,' <PrMtid>>>'
      return

      entry print_tides_opt (opt)
c
c Here there are a few options, some a bit excentric
c D means default (usually what the minus-option reverts to)
c 
c +H  - print relative phase w.r.t. tide potential (-H: absolute, D)
c +O  - change lead to lag (-O: positive means lead)
c +P  - include the potential's phase in the list (-P: don't, D)
c +Q  - If there is one column to print in ZDR, the potential's amp,pha
c       will appear in column 1 and the effect's in column 2;
c       else (i.e. if the effect is in column 2 and 3) column 1 will be
c       overwritten with the potential's amp,pha.
c -L  - evaluate the number of 90-degree shifts of the effect
c       w.r.t. the potential's phase, and reduce for site longitude. 
c +L  - print floating point phase shifts, including the shift 
c       due to site longitude (D
c
c For option -L, the site longitude must be submitted using
c       Call Print_Tides_Xs (longitude_in_radians)
c 

      if (q_index(opt,'+h',iqq).or.q_index(opt,'+H',iqq)) qp=.true.
      if (q_index(opt,'-h',iqq).or.q_index(opt,'-H',iqq)) qp=.false.
      if (q_index(opt,'+o',iqq).or.q_index(opt,'+O',iqq)) qo=.true.
      if (q_index(opt,'-o',iqq).or.q_index(opt,'-O',iqq)) qo=.false.
      if (q_index(opt,'+p',iqq).or.q_index(opt,'+P',iqq)) qpp=.true.
      if (q_index(opt,'-p',iqq).or.q_index(opt,'-P',iqq)) qpp=.false.
      if (q_index(opt,'+q',iqq).or.q_index(opt,'+Q',iqq)) qp1=.true.
      if (q_index(opt,'-q',iqq).or.q_index(opt,'-Q',iqq)) qp1=.false.
      if (q_index(opt,'-l',iqq).or.q_index(opt,'-L',iqq)) then
         qphint=.true.
      endif
      if (index(opt,'O').gt.0
     &   .and.index(opt,'+O').eq.0.and.index(opt,'-O').eq.0) qo=.true.
      if (index(opt,'P').gt.0
     &   .and.index(opt,'+P').eq.0.and.index(opt,'-P').eq.0) qp=.true.

      print '(a,l2)',' <PrTOpt>>> Reduced phase:',qp
      print '(a,l2)',' <PrTOpt>>> Print phase of potential:',qpp
      print '(a,l2)',' <PrTOpt>>> Oceanographers phase:',qo
      print '(a,f10.5)',' <PrTOpt>>> Site longitude ',slon/rad
      return

      entry print_tides_xs (xx)
      slon=xx
      return
      end
